from enum import Enum#


class SALUSInformation:
    class UpdateState(Enum):
        NO_UPDATE_AVAILABLE = 0
        UPDATE_AVAILABLE = 1

    def __init__(self, api, raw_information):
        self._api = api
        self._raw_information = raw_information

    @property
    def gateway(self):
        return next(filter(lambda x: not x["data"]["DeviceType"] == 300, self._raw_information))

    @property
    def version(self):
        return self.gateway.get("sOTA", {}).get("OTAFirmwareVersion_d", "n/a")

    @property
    def update_state(self) -> UpdateState:
        return self.UpdateState(self.gateway.get("sOTA", {}).get("OTAFirmwareVersion_d", 0))

    @property
    def gateway_raw(self):
        return None

    @property
    def other_devices_raw(self):
        return list(
            filter(
                lambda x: not x["data"]["DeviceType"] == 200 and not x["data"]["DeviceType"] == 300,
                self._raw_information
            )
        )

    def summary(self):
        print(f"Information:")
        print(f"  SW Version         : {self.version}")
        print(f"  State              : {self.update_state}")



import logging

import voluptuous as vol
from salusshcpy import SALUSSession
from .const import DOMAIN, CONF_EUID

from homeassistant import config_entries, core, exceptions
from homeassistant.const import CONF_IP_ADDRESS, CONF_NAME

DATA_SCHEMA = vol.Schema(
    {
        vol.Required(CONF_NAME, default="SALUSSHC"): str,
        vol.Required(CONF_IP_ADDRESS): str,
        vol.Required(CONF_EUID): str
    }
)

_LOGGER = logging.getLogger(__name__)


async def validate_input(hass: core.HomeAssistant, data):
    session = await hass.async_add_executor_job(
        SALUSSession,
        data[CONF_IP_ADDRESS],
        data[CONF_EUID]
    )
    status = session.information.version

    if status == "n/a":
        raise InvalidAuth

    return {"title": data[CONF_NAME]}


class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
    VERSION = 1
    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_POLL

    async def async_step_user(self, user_input=None):
        errors = {}

        if user_input is not None:
            try:
                info = await validate_input(self.hass, user_input)

                await self.async_set_unique_id(user_input[CONF_IP_ADDRESS])
                self._abort_if_unique_id_configured()

                return self.async_create_entry(title=info["title"], data=user_input)
            except CannotConnect:
                errors["base"] = "cannot_connect"
            except InvalidAuth:
                errors["base"] = "invalid_auth"
            except Exception:  # pylint: disable=broad-except
                _LOGGER.exception("Unexpected exception")
                errors["base"] = "unknown"

        return self.async_show_form(
            step_id="user", data_schema=DATA_SCHEMA, errors=errors
        )

    async def async_step_import(self, user_input):
        await self.async_set_unique_id(user_input[CONF_IP_ADDRESS])
        self._abort_if_unique_id_configured()

        return await self.async_step_user(user_input)


class CannotConnect(exceptions.HomeAssistantError):
    """Error to indicate we cannot connect."""


class InvalidAuth(exceptions.HomeAssistantError):
    """Error to indicate there is invalid auth."""



import json
import logging
import time

import requests
from .encryptor import IT600Encryptor

from .const import (
    HVAC_MODE_HEAT,
    HVAC_MODE_OFF,
    PRESET_OFF,
    PRESET_PERMANENT_HOLD
)

logger = logging.getLogger("salsusshcpy")


class JSONRPCError(Exception):
    def __init__(self, code, message):
        super().__init__()
        self._code = code
        self._message = message

    @property
    def code(self):
        return self._code

    @property
    def message(self):
        return self._message

    def __str__(self):
        return f"JSONRPCError (code: {self.code}, message: {self.message})"


class SHCAPI:
    def __init__(self, controller_ip: str, euid: str):
        self._controller_ip = controller_ip
        self._euid = euid
        self._api_root = f"http://{self._controller_ip}:80/deviceid"
        self._encryptor = IT600Encryptor(euid)

        self._requests_session = requests.Session()
        self._requests_session.headers.update(
            {"Content-Type": "application/json"}
        )
        self._requests_session.verify = False

        import urllib3

        urllib3.disable_warnings()

    @property
    def api_url(self):
        return self._api_root

    def _get_api_result_or_fail(
        self, api_url, expected_type=None, expected_element_type=None, headers=None, timeout=30
    ):
        try:
            result = self._requests_session.get(api_url, headers=headers, timeout=timeout)
            print(result)
            if not result.ok:
                self._process_nok_result(result)

            else:
                if len(result.content) > 0:
                    result = json.loads(result.content)
                    if expected_type is not None:
                        assert result["@type"] == expected_type
                    if expected_element_type is not None:
                        for result_ in result:
                            assert result_["@type"] == expected_element_type

                    return result
                else:
                    return {}
        except requests.exceptions.SSLError as e:
            raise Exception(f"API call returned SSLError: {e}.")

    def _post_api_or_fail(self, api_url, body, timeout=30):
        response = self._requests_session.post(api_url, data=self._encryptor.encrypt(json.dumps(body)), timeout=timeout)
        response_json_string = self._encryptor.decrypt(response.content)
        result = json.loads(response_json_string)

        if not result["status"] == "success":
            self._process_nok_result(response)
        if not result["id"] is None:
            return result["id"]
        else:
            return {}

    def _process_nok_result(self, result):
        print(f"Body: {result.request.body}")
        print(f"Headers: {result.request.headers}")
        print(f"URL: {result.request.url}")

        raise ValueError(
            f"API call returned non-OK result (code {result.status_code})!: {result.content}"
        )

    def get_information(self, wait_seconds=0):
        api_url = f"{self._api_root}/read"
        data = {"requestAttr": "readall"}

        try:
            result = self._post_api_or_fail(api_url, data, wait_seconds + 5)
        except Exception as e:
            logging.error(f"Failed to get information from SALUS controller: {e}")
            return None
        return result

    def long_polling_poll(self, wait_seconds=0):
        api_url = f"{self._api_root}/read"
        data = {"requestAttr": "readall"}

        try:
            result = self._post_api_or_fail(api_url, data, wait_seconds + 5)
            time.sleep(wait_seconds + 5)
        except Exception as e:
            logging.error(f"Failed to start long polling for SALUS Controller: {e}")
            return None
        return result

    async def set_temperature(self, device, celsius: float):
        api_url = f"{self._api_root}/write"
        data = {
                "requestAttr": "write",
                "id": [
                    {
                        "data": device.data,
                        "sIT600TH": {"SetHeatingSetpoint_x100": int((round(celsius * 2) / 2) * 100)}
                    }
                ]
            }

        try:
            result = self._post_api_or_fail(api_url, data)
        except Exception as e:
            logging.error(f"Failed to set temperature for device with id: {device.unique_id}")
            return None
        return result

    async def set_climate_device_mode(self, device, mode: str):
        api_url = f"{self._api_root}/write"
        data = {
                "requestAttr": "write",
                "id": [
                    {
                        "data": device.data,
                        "sIT600TH": {"SetHoldType": 7 if mode == HVAC_MODE_OFF else 0}
                    }
                ]
            }
        try:
            result = self._post_api_or_fail(api_url, data)
        except Exception as e:
            logging.error(f"Failed to set mode for device with id: {device.unique_id}")
            return None
        return result

    async def set_climate_device_preset(self, device, preset: str):
        api_url = f"{self._api_root}/write"
        data = {
                "requestAttr": "write",
                "id": [
                    {
                        "data": device.data,
                        "sIT600TH": {
                            "SetHoldType": 7 if preset == PRESET_OFF else 2 if preset == PRESET_PERMANENT_HOLD else 0
                        }
                    }
                ],
            }
        try:
            result = self._post_api_or_fail(api_url, data)
        except Exception as e:
            logging.error(f"Failed to set preset for device with id: {device.unique_id}")
            return None
        return result




# Temperature units
TEMP_CELSIUS = "°C"

# Supported features
SUPPORT_TARGET_TEMPERATURE = 1
SUPPORT_PRESET_MODE = 16

# HVAC modes
HVAC_MODE_OFF = "off"
HVAC_MODE_HEAT = "heat"

# HVAC states
CURRENT_HVAC_OFF = "off"
CURRENT_HVAC_HEAT = "heating"
CURRENT_HVAC_IDLE = "idle"

# Supported presets
PRESET_FOLLOW_SCHEDULE = "Follow Schedule"
PRESET_PERMANENT_HOLD = "Permanent Hold"
PRESET_OFF = "Off"


import logging

from salusshcpy import SALUSSession, SALUSThermostat
from homeassistant.components.climate import ClimateEntity
from homeassistant.const import ATTR_TEMPERATURE

from . import DOMAIN

_LOGGER = logging.getLogger(__name__)


async def async_setup_entry(hass, config_entry, async_add_entities):
    entities = []
    session: SALUSSession = hass.data[DOMAIN][config_entry.entry_id]

    for device in session.device_helper.thermostats:

        entity = Thermostat(device)
        entities += [entity]

    if entities:
        async_add_entities(entities)


class Thermostat(ClimateEntity):

    def __init__(
        self,
        device: SALUSThermostat
    ):
        self._name = device.name
        self._unique_id = device.unique_id
        self._device = device

    async def async_added_to_hass(self):
        await super().async_added_to_hass()

        def on_state_changed():
            self.schedule_update_ha_state()

        self._device.subscribe_callback(self.entity_id, on_state_changed)

    async def async_will_remove_from_hass(self):
        await super().async_will_remove_from_hass()
        self._device.unsubscribe_callback(self.entity_id)

    @property
    def should_poll(self):
        return False

    @property
    def name(self):
        return self._name

    @property
    def unique_id(self):
        return self._unique_id

    @property
    def temperature_unit(self):
        return self._device.temperature_unit

    @property
    def current_temperature(self):
        return self._device.current_temperature

    @property
    def max_temp(self):
        return self._device.max_temp

    @property
    def min_temp(self):
        return self._device.min_temp

    @property
    def target_temperature(self):
        return self._device.target_temperature

    @property
    def target_temperature_step(self):
        return self._device.precision

    @property
    def hvac_mode(self):
        return self._device.hvac_mode

    @property
    def hvac_modes(self):
        return self._device.hvac_modes

    @property
    def hvac_action(self):
        return self._device.hvac_action

    @property
    def preset_mode(self):
        return self._device.preset_mode

    @property
    def preset_modes(self):
        return self._device.preset_modes

    @property
    def supported_features(self):
        return self._device.supported_features

    async def async_set_temperature(self, **kwargs):
        temperature = kwargs.get(ATTR_TEMPERATURE)
        if temperature is None:
            return

        if self.min_temp <= temperature <= self.max_temp:
            self._device.set_temperature(float(temperature))

    async def async_set_hvac_mode(self, hvac_mode: str):
        if hvac_mode not in self.hvac_modes:
            return

        self._device.set_climate_device_mode(hvac_mode)

    async def async_set_preset_mode(self, preset_mode: str):
        if preset_mode not in self.preset_modes:
            return

        self._device.set_climate_device_preset(preset_mode)



import asyncio
import logging

from salusshcpy import SALUSSession

from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
from homeassistant.core import HomeAssistant
from homeassistant.const import (
    CONF_IP_ADDRESS,
    CONF_NAME,
    EVENT_HOMEASSISTANT_START,
    EVENT_HOMEASSISTANT_STOP,
)

from homeassistant.helpers import device_registry as dr

from .const import (
    DOMAIN,
    CONF_EUID
)

PLATFORMS = [
    "thermostat"
]

_LOGGER = logging.getLogger(__name__)


async def async_setup(hass: HomeAssistant, config: dict):
    hass.data.setdefault(DOMAIN, {})
    conf = config.get(DOMAIN)

    if not conf:
        return True

    configured_hosts = {
        entry.data.get("ip_address")
        for entry in hass.config_entries.async_entries(DOMAIN)
    }

    if conf[CONF_IP_ADDRESS] in configured_hosts:
        return True

    hass.async_create_task(
        hass.config_entries.flow.async_init(
            DOMAIN, context={"source": SOURCE_IMPORT}, data=conf
        )
    )
    return True


async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
    data = entry.data

    _LOGGER.cr("Connecting to SALUS Gateway")

    session = await hass.async_add_executor_job(
        SALUSSession,
        data[CONF_IP_ADDRESS],
        data[CONF_EUID]
    )

    information = session.information
    hass.data[DOMAIN][entry.entry_id] = session
    device_registry = await dr.async_get_registry(hass)
    device_registry.async_get_or_create(
        config_entry_id=entry.entry_id,
        identifiers={(DOMAIN, data[CONF_IP_ADDRESS])},
        manufacturer="SALUS",
        name=data[CONF_NAME],
        model="Gateway",
        sw_version=information.version,
    )

    for component in PLATFORMS:
        hass.async_create_task(
            hass.config_entries.async_forward_entry_setup(entry, component)
        )

    async def stop_polling(event):
        _LOGGER.debug("Stopping polling service of SALUS")
        await hass.async_add_executor_job(session.stop_polling)

    async def start_polling(event):
        """Start polling service."""
        _LOGGER.debug("Starting polling service of SALUS")
        await hass.async_add_executor_job(session.start_polling)
        session.reset_connection_listener = hass.bus.async_listen_once(
            EVENT_HOMEASSISTANT_STOP, stop_polling
        )

    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_START, start_polling)

    return True


async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
    session: SALUSSession = hass.data[DOMAIN][entry.entry_id]
    session.reset_connection_listener()
    _LOGGER.debug("Stopping polling service of SALUS")
    await hass.async_add_executor_job(session.stop_polling)

    unload_ok = all(
        await asyncio.gather(
            *[
                hass.config_entries.async_forward_entry_unload(entry, component)
                for component in PLATFORMS
            ]
        )
    )

    if unload_ok:
        hass.data[DOMAIN].pop(entry.entry_id)

    return unload_ok



from setuptools import find_packages, setup

setup(
    name="salusshcpy",
    version="0.0.2",
    author="Emre Bolat",
    author_email="emre-bolat@hotmail.de",
    description="Salus Smart Home Controller API Python Library",
    license='bsd-3-clause',
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        "License :: OSI Approved :: BSD License"
    ],
    platform='any',
    python_requires='>=3.7',
    install_requires=['requests>=2.22']
)


import logging

from salusshcpy import SALUSDevice

from homeassistant.helpers.entity import Entity

from .const import DOMAIN

_LOGGER = logging.getLogger(__name__)


class SALUSEntity(Entity):

    def __init__(self, device: SALUSDevice, controller_ip: str):
        self._device = device
        self._controller_ip = controller_ip

    async def async_added_to_hass(self):
        await super().async_added_to_hass()

        def on_state_changed():
            self.schedule_update_ha_state()
            self._device.subscribe_callback(self.entity_id, on_state_changed)

    async def async_will_remove_from_hass(self):
        await super().async_will_remove_from_hass()
        self._device.unsubscribe_callback(self.entity_id)

    @property
    def should_poll(self):
        return False

    def update(self):
        self._device.update()

    @property
    def unique_id(self):
        return self._device.unique_id

    @property
    def is_available(self):
        return self._device.is_available

    @property
    def name(self):
        return self._device.name

    @property
    def device_info(self):
        """Return the device info."""
        return {
            "identifiers": {(DOMAIN, self.unique_id)},
            "name": self.name,
            "manufacturer": "SALUS",
            "model": self._device.model,
            "sw_version": "",
            "via_device": (DOMAIN, self._controller_ip),
        }



from .device import SALUSDevice
from .api import SHCAPI

from .const import (
    SUPPORT_PRESET_MODE,
    SUPPORT_TARGET_TEMPERATURE,
    TEMP_CELSIUS,
    CURRENT_HVAC_HEAT,
    CURRENT_HVAC_IDLE,
    CURRENT_HVAC_OFF,
    HVAC_MODE_HEAT,
    HVAC_MODE_OFF,
    PRESET_FOLLOW_SCHEDULE,
    PRESET_OFF,
    PRESET_PERMANENT_HOLD,
)


class SALUSThermostat(SALUSDevice):

    def __init__(self, api: SHCAPI, raw_device):
        super().__init__(api, raw_device)

    @property
    def device_class(self):
        return "temperature"

    @property
    def supported_features(self):
        return SUPPORT_TARGET_TEMPERATURE | SUPPORT_PRESET_MODE

    @property
    def temperature_unit(self):
        return TEMP_CELSIUS

    @property
    def precision(self):
        return 0.5

    @property
    def current_temperature(self):
        return self.sit_600th["LocalTemperature_x100"] / 100

    @property
    def target_temperature(self):
        return self.sit_600th["HeatingSetpoint_x100"] / 100

    @property
    def max_temp(self):
        return self.sit_600th.get("MaxHeatSetpoint_x100", 3500) / 100

    @property
    def min_temp(self):
        return self.sit_600th.get("MinHeatSetpoint_x100", 500) / 100

    @property
    def hvac_mode(self):
        return HVAC_MODE_OFF if self.sit_600th["HoldType"] == 7 else HVAC_MODE_HEAT

    @property
    def hvac_action(self):
        return CURRENT_HVAC_OFF if self.sit_600th["HoldType"] == 7 else CURRENT_HVAC_IDLE \
            if self.sit_600th["RunningState"] % 2 == 0 else CURRENT_HVAC_HEAT

    @property
    def hvac_modes(self):
        return [HVAC_MODE_OFF, HVAC_MODE_HEAT]

    @property
    def preset_mode(self):
        return PRESET_OFF if self.sit_600th["HoldType"] == 7 else PRESET_PERMANENT_HOLD if self.sit_600th["HoldType"] == 2 \
            else PRESET_FOLLOW_SCHEDULE

    @property
    def preset_modes(self):
        return [PRESET_FOLLOW_SCHEDULE, PRESET_PERMANENT_HOLD, PRESET_OFF]

    @property
    def mac_address(self):
        return self.s_zdo.get("MACAddress", None)

    @property
    def firmware_version(self):
        return self.s_zdo.get("FirmwareVersion", None)

    def update(self):
        print(f"Update Thermostat")

    def summary(self):
        print(f"TRV Thermostat:")
        super().summary()
        print(f"  Current temperature:" + str(self.current_temperature))
        print(f"  Target temperature:" + str(self.target_temperature))

    def process_long_polling_poll_result(self, raw_result):
        super().process_long_polling_poll_result(raw_result)
        self.summary()

    async def set_temperature(self, celsius: float):
        await self.api.set_temperature(self, celsius)

    async def set_climate_device_mode(self, mode: str):
        await self.api.set_climate_device_mode(self, mode)

    async def set_climate_device_preset(self, preset: str):
        await self.api.set_climate_device_preset(self, preset)


MODEL_MAPPING = {
    "SQ610RF": SALUSThermostat
}

SUPPORTED_MODELS = MODEL_MAPPING.keys()


def build(api, raw_device):
    device_model = raw_device["DeviceL"]["ModelIdentifier_i"]
    assert device_model in SUPPORTED_MODELS, "Device model is supported"
    return MODEL_MAPPING[device_model](api=api, raw_device=raw_device)
